/* Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <viewmgr.h>
#include <inputmgr.h>
#include <app_debug.h>

#include "define.h"
#include "utils.h"

enum button_type {
	BTN_LIVETV = 0,
	BTN_FAVORITE,
	BTN_LOCK,
	BTN_SHARE,
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *live_btn;
	Evas_Object *menu_btn[COUNT_ACTION];
};

struct _action_info {
	int id;
	const char *title;
	const char *style;
	Eina_Bool disable;
};

static struct _action_info action_info[] = {
	{
		.id = BTN_FAVORITE,
		.title = STR_FAVORITE,
		.style = STYLE_ACTION_MENU_FAV_BTN,
		.disable = EINA_FALSE
	},
	{
		.id = BTN_LOCK,
		.title = STR_LOCK,
		.style = STYLE_ACTION_MENU_LOCK_BTN,
		.disable = EINA_TRUE
	},
	{
		.id = BTN_SHARE,
		.title = STR_SHARE,
		.style = STYLE_ACTION_MENU_SHARE_BTN,
		.disable = EINA_TRUE
	}
};

static void _mouse_move_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	if (!obj) {
		_ERR("Invalid argument.");
		return;
	}

	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static void _favorite_selected(void *data, Evas_Object *obj)
{
	/* It will be implemented later */
}

static void _livetv_selected(void *data, Evas_Object *obj)
{
	/* It will be implemented later */
}

static void _clicked_cb(int id, void *data, Evas_Object *obj)
{

	switch (id) {
	case BTN_LIVETV:
		_livetv_selected(data, obj);
		break;

	case BTN_FAVORITE:
		_favorite_selected(data, obj);
		break;

	case BTN_LOCK:
		/* It is not supported yet. */
		break;

	case BTN_SHARE:
		/* It is not supported yet. */
		break;

	default:
		_ERR("Unhandled action type.");
	}
}

static void _key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	if (!ev) {
		_ERR("Invalid argument.");
		return;
	}

	if (!strcmp(ev->keyname, KEY_BACK)
			|| !strcmp(ev->keyname, KEY_BACK_REMOTE))
		viewmgr_hide_view(VIEW_ID_ACTION_MENU);
}

static input_handler _btn_input_handler = {
	.mouse_move = _mouse_move_cb,
	.clicked = _clicked_cb,
	.key_down = _key_down_cb
};

static bool _draw_livetv_area(struct _priv *priv)
{
	Evas_Object *btn;

	if (!priv || !priv->base) {
		_ERR("Invalid argument.");
		return false;
	}

	btn = utils_add_button(priv->base,
			PART_LIVETV_BTN, STR_LIVETV, STYLE_LIVETV_BTN);
	if (!btn) {
		_ERR("Add button failed.");
		return false;
	}

	inputmgr_add_callback(btn, BTN_LIVETV, &_btn_input_handler, priv);

	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_LEFT);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_RIGHT);

	priv->live_btn = btn;

	return true;
}

static bool _draw_menu_area(struct _priv *priv)
{
	Evas_Object *table, *btn;
	int i, row = 0, col = 0;

	if (!priv || !priv->base) {
		_ERR("Invalid argument.");
		return false;
	}

	table = utils_add_table(priv->base, PART_ACTION_MENU,
			SIZE_ACTION_TABLE_BORDER, SIZE_ACTION_TABLE_BORDER);
	if (!table) {
		_ERR("Add table failed.");
		return false;
	}

	for (i = 0; i < COUNT_ACTION; i++) {
		btn = utils_add_button(priv->base, NULL, action_info[i].title,
				action_info[i].style);
		if (!btn) {
			_ERR("Add button failed.");
			evas_object_del(table);
			return false;
		}

		inputmgr_add_callback(btn, action_info[i].id,
				&_btn_input_handler, priv);

		if (action_info[i].disable)
			elm_object_disabled_set(btn, EINA_TRUE);

		col = i % SIZE_COL_MAX;
		row = i / SIZE_COL_MAX;

		elm_table_pack(table, btn, col, row, 1, 1);

		priv->menu_btn[i] = btn;
	}

	/* 'Favorite' can only be focused now. */
	elm_object_focus_next_object_set(priv->menu_btn[0],
			priv->menu_btn[0], ELM_FOCUS_LEFT);
	elm_object_focus_next_object_set(priv->menu_btn[0],
			priv->menu_btn[0], ELM_FOCUS_RIGHT);
	elm_object_focus_next_object_set(priv->menu_btn[0],
			priv->menu_btn[0], ELM_FOCUS_DOWN);
	elm_object_focus_next_object_set(priv->live_btn, priv->menu_btn[0],
			ELM_FOCUS_DOWN);

	return true;
}

static bool _draw_favorite_area(struct _priv *priv)
{
	if (!priv || !priv->base) {
		_ERR("Invalid argument.");
		return false;
	}

	/* It will be implemented later */

	return true;
}

static void _draw_view_content(struct _priv *priv)
{
	if (!priv) {
		_ERR("Invalid argument.");
		return;
	}

	if (!_draw_livetv_area(priv)) {
		_ERR("Draw livetv area failed.");
		return;
	}

	if (!_draw_menu_area(priv)) {
		_ERR("Draw menu area failed.");
		return;
	}

	if (!_draw_favorite_area(priv))
		_ERR("Draw favorite area failed.");
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;

	if (!win) {
		_ERR("Get window object failed.");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("Calloc failed.");
		return NULL;
	}

	base = utils_add_layout(win, GRP_VIEW_ACTION_MENU);
	if (!base) {
		_ERR("Add layout failed.");
		free(priv);
		return NULL;
	}

	priv->win = win;
	priv->base = base;

	if (!viewmgr_set_view_data(VIEW_ID_ACTION_MENU, priv)) {
		_ERR("Set view data failed.");
		evas_object_del(base);
		free(priv);
		return NULL;
	}

	_draw_view_content(priv);

	return base;
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Get data failed.");
		return;
	}
	priv = (struct _priv *)data;

	if (priv->base)
		evas_object_show(priv->base);

	elm_object_focus_set(priv->live_btn, EINA_TRUE);
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Get data failed.");
		return;
	}
	priv = (struct _priv *)data;

	if (priv->base)
		evas_object_hide(priv->base);
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Get data failed.");
		return;
	}
	priv = (struct _priv *)data;

	if (priv->base)
		evas_object_del(priv->base);

	free(priv);
}

static view_class _vclass = {
	.view_id = VIEW_ID_ACTION_MENU,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
};

view_class *view_action_menu_get_vclass(void)
{
	return &_vclass;
}
